﻿Imports System.Windows.Forms
Imports ActiveDevelop.Wpf.Imaging
Imports System.IO
Imports System.Threading.Tasks


Class MainWindow

    Private Sub MainWindow_Loaded(ByVal sender As Object,
                                  ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        'Preselect jPeg as default file format.
        cbImageFormat.SelectedIndex = 0

        'MyImages + ImageConverter + current date = default folder name
        Dim myOutputFolder As String
        myOutputFolder = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)
        myOutputFolder &= "\ImageConverter\" & Now.ToString("yyyy-MM-dd")
        tbOutputFolder.Text = myOutputFolder
    End Sub

    Private Sub MenuItemBeenden_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        Me.Close()
    End Sub

    Private Sub ChooseOutputFolderButton_Click(ByVal sender As System.Object,
                                            ByVal e As System.Windows.RoutedEventArgs)

        Dim folderBrowser = New FolderBrowserDialog

        'New folders can be created.
        folderBrowser.ShowNewFolderButton = True
        'Set the dialog title.
        folderBrowser.Description = "Choose target folder:"

        'Show the dialog.
        Dim dr = folderBrowser.ShowDialog
        If dr = Forms.DialogResult.OK Then
            tbOutputFolder.Text = folderBrowser.SelectedPath
        End If
    End Sub

    'Wire up the event with handles:
    Private Sub AddImagesButton_Click(ByVal sender As System.Object,
                                      ByVal e As System.Windows.RoutedEventArgs) Handles btnAddImage.Click

        'We use OpenFileDialog from Win32, because it is "made" for WPF.
        'We have to use the fully qualified name, though, since it is ambigious due to .Forms and .Win32.
        Dim fod = New Microsoft.Win32.OpenFileDialog
        'Only existing files and folders can be selected.
        fod.CheckPathExists = True
        fod.CheckFileExists = True
        'We can select more than one file.
        fod.Multiselect = True
        'Let's filter for image file types:
        fod.Filter = "JPeg files (*.jpg; *.jpeg)|*.jpg; *.jpeg" &
                    "|Bitmap files (*.bmp)|*.bmp" &
                    "|TIFF files (*.tif;*.tiff)|*.tif;*.tiff" &
                    "|PNG files (*.png)|*.png" &
                    "|All files (*.*)|*.*"
        'Accept only valid filenames.
        fod.ValidateNames = True
        'Default filter is jPeg:
        fod.DefaultExt = "*.jpg"

        'Remember the return value into dr.
        Dim dr = fod.ShowDialog
        'Check for success.
        If dr = True Then
            Array.ForEach(fod.FileNames, Sub(filename)
                                             lbFileNames.Items.Add(filename)
                                         End Sub)
        End If
    End Sub

    Private Sub StartButton_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)

        'Instanciate and show the preview window
        Dim picViewer As New BitmapViewer
        picViewer.Show()

        'Read new width.
        Dim myReduceToX = Integer.Parse(tbNewXResolution.Text)
        'Dateityp einlesen
        Dim newExtension = DirectCast(cbImageFormat.SelectedItem, ComboBoxItem).Content.ToString

        'Retrieve count of calculations and assign that to progress bar.
        Me.TotalProgressBar.Maximum = lbFileNames.Items.Count
        'counter for done calculations = progress.
        Dim progressCount = 0

        'For each image...
        For Each filenameItem As String In lbFileNames.Items
            'show its filename
            CurrentFileTextBlock.Text = filenameItem
            'and save.
            Dim fileInfo As New FileInfo(filenameItem)

            'instanciate Manager and create a copy of the picture.
            Dim currentPicture As New WriteableBitmapManager(filenameItem)
            Dim mySmallWbm = currentPicture.CreateCompatible(myReduceToX)
            'Calculate the downsizing factor.
            Dim pixelFakt As Double = currentPicture.WriteableBitmap.PixelWidth / myReduceToX

            'Geht the stopwatch in gear.
            Dim sw = Stopwatch.StartNew
            'Downsize the image.
            For y = 0 To mySmallWbm.WriteableBitmap.PixelHeight - 1
                For x = 0 To myReduceToX - 1
                    mySmallWbm.SetPixel(x, y,
                       currentPicture.GetPixel(CInt(Math.Truncate(pixelFakt * x)),
                                                CInt(Math.Truncate(pixelFakt * y))))
                Next
            Next

            'and update.
            mySmallWbm.UpdateBitmap()

            'Stop clock.
            sw.Stop()

            'Show the resized image.
            picViewer.ShowPicture(mySmallWbm, "Duration of resizing in ms: " & sw.ElapsedMilliseconds.ToString("#,##0.00"))

            'Naaasty workaround: see more in the threading chapter.
            System.Windows.Forms.Application.DoEvents()

            'Adjust the file type.
            Dim newFilename = fileInfo.Name.Replace(fileInfo.Extension, newExtension)
            'Take folder and postfix into account.
            newFilename = tbOutputFolder.Text &
                          tbPostfix.Text & "\" & newFilename
            'Save.
            mySmallWbm.SaveBitmap(newFilename)

            'And show progress.
            progressCount += 1
            TotalProgressBar.Value = progressCount

        Next

        picViewer.Close()
    End Sub
End Class
