Public NotInheritable Class DateCalcHelper

    ''' <summary>
    ''' Calculates the first day of a month.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function FirstDayOfMonth(ByVal CurrentDate As Date) As Date
        Return New Date(CurrentDate.Year, CurrentDate.Month, 1)
    End Function

    ''' <summary>
    ''' Calculates the last day of a month.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function LastDayOfMonth(ByVal CurrentDate As Date) As Date
        Return New Date(CurrentDate.Year, CurrentDate.Month, 1).AddMonths(1).AddDays(-1)
    End Function

    ''' <summary>
    ''' Calculates the first day of the year.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function FirstOfYear(ByVal CurrentDate As Date) As Date
        Return New Date(CurrentDate.Year, 1, 1)
    End Function

    ''' <summary>
    ''' Calculates the Monday of the first week of the month.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function MondayOfFirstWeekOfMonth(ByVal CurrentDate As Date) As Date
        Dim locDate As Date = FirstDayOfMonth(CurrentDate)
        If Weekday(locDate) = DayOfWeek.Monday Then
            Return locDate
        End If
        Return locDate.AddDays(6 - Weekday(CurrentDate))
    End Function

    ''' <summary>
    ''' Calculates the Monday of the week.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function MondayOfWeek(ByVal CurrentDate As Date) As Date
        If Weekday(CurrentDate) = DayOfWeek.Monday Then
            Return CurrentDate
        Else
            Return CurrentDate.AddDays(-Weekday(CurrentDate) + 1)
        End If
    End Function

    ''' <summary>
    ''' Calculates the Monday of the second week of the month.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function MondayOfSecondWeekOfMonth(ByVal currentDate As Date) As Date
        Return MondayOfFirstWeekOfMonth(currentDate).AddDays(7)
    End Function

    ''' <summary>
    ''' Calculates the monday of the last week of the month.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function MondayOfLastWeekOfMonth(ByVal CurrentDate As Date) As Date
        Dim locDate As Date = FirstDayOfMonth(CurrentDate).AddDays(-1)
        If Weekday(locDate) = DayOfWeek.Monday Then
            Return locDate
        End If
        Return locDate.AddDays(-Weekday(CurrentDate) + 1)
    End Function

    ''' <summary>
    ''' Calculates the upcoming workday.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <param name="WorkOnSaturdays">True, if Saturday is a workday.</param>
    ''' <param name="WorkOnSundays">True, if Sunday is a workday.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function NextWorkday(ByVal CurrentDate As Date, ByVal WorkOnSaturdays As Boolean, ByVal WorkOnSundays As Boolean) As Date
        CurrentDate = CurrentDate.AddDays(1)
        If Weekday(CurrentDate) = DayOfWeek.Saturday And Not WorkOnSaturdays Then
            CurrentDate = CurrentDate.AddDays(1)
        End If
        If Weekday(CurrentDate) = DayOfWeek.Sunday And Not WorkOnSundays Then
            CurrentDate = CurrentDate.AddDays(1)
        End If
        Return CurrentDate
    End Function

    ''' <summary>
    ''' Calculates the previous workday.
    ''' </summary>
    ''' <param name="CurrentDate">Date to base the calculation on.</param>
    ''' <param name="WorkOnSaturdays">True, if Saturday is a workday.</param>
    ''' <param name="WorkOnSundays">True, if Sunday is a workday.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function PreviousWorkday(ByVal CurrentDate As Date, ByVal WorkOnSaturdays As Boolean, ByVal WorkOnSundays As Boolean) As Date
        CurrentDate = CurrentDate.AddDays(-1)
        If Weekday(CurrentDate) = DayOfWeek.Sunday And Not WorkOnSundays Then
            CurrentDate = CurrentDate.AddDays(-1)
        End If
        If Weekday(CurrentDate) = DayOfWeek.Saturday And Not WorkOnSaturdays Then
            CurrentDate = CurrentDate.AddDays(-1)
        End If
        Return CurrentDate
    End Function
End Class
