Imports System.IO

Module mdlMain

    'The master data file "inventory.txt" has to be in the app's folder.
    Private Filename As String = My.Application.Info.DirectoryPath & "\Inventory.txt"

    Sub Main()

        'Streamreader to read the file.
        Dim locSr As StreamReader
        Dim locList As New DynamicList  ' The dynamic List.
        Dim locElements() As String     ' The shop items.
        Dim locShopItem As ShopItem     ' A single shop item.
        Dim locDisplayMode As PrintType ' Display mode.

        'Let's test, if the inventory file exist.
        Try
            locSr = New StreamReader(Filename, System.Text.Encoding.Default)
        Catch ex As Exception
            Console.WriteLine("Error while reading the inventory file!" & _
                              vbNewLine & ex.Message)
            Console.WriteLine()
            Console.WriteLine("Press any key to quit.")
            Console.ReadKey()
            Exit Sub
        End Try

        Console.WriteLine("Choose (1) for brief or (2) for detailed display.")
        Dim locKey As Char = Console.ReadKey.KeyChar
        If locKey = "1"c Then
            locDisplayMode = PrintType.Brief
        Else
            locDisplayMode = PrintType.Detailed
        End If

        Do
            Try
                'Read a line.
                Dim locLine As String = locSr.ReadLine()

                'Nothing entered, that's it then.
                If String.IsNullOrEmpty(locLine) Then
                    locSr.Close()
                    Exit Do
                End If

                'Skip the semicolumn.
                If Not locLine.StartsWith(";") Then

                    'This is how we split the item's line to several array elements.!
                    locElements = locLine.Split(New Char() {";"c})

                    If locElements(FieldOrder.Type) = "1" Then
                        locShopItem = New BookItem(locElements)
                    Else
                        locShopItem = New DVDItem(locElements)
                    End If
                    locList.Add(locShopItem)
                End If

            Catch ex As Exception
                Console.WriteLine("Error while parsing the inventory file!" & _
                                  vbNewLine & ex.Message)
                Console.WriteLine()
                Console.WriteLine("Press any key to quit.")
                Console.ReadKey()
                locSr.Close()
                Exit Sub
            End Try

        Loop

        Dim locGrossAmount As Double = 0

        'Print out every item.
        For count As Integer = 0 To locList.Count - 1
            locList(count).PrintTypeSetting = locDisplayMode
            Console.WriteLine(locList(count).ToString())
            locGrossAmount += locList(count).GrossPrice
        Next

        Console.WriteLine()
        Console.WriteLine("-----------------------------")
        Console.WriteLine("Gross amount: " & locGrossAmount.ToString("#,##0.00") & " $")
        Console.WriteLine("-----------------------------")
        Console.WriteLine("-----------------------------")
        Console.WriteLine()
        Console.WriteLine("Press enter to finish.")
        Console.ReadLine()

    End Sub

End Module

Enum PrintType  ' Reportform
    Brief
    Detailed
End Enum

Enum FieldOrder ' Responsible for the field order.
    Type
    OrderNumber
    Titel
    AdditionalRemarks1
    GrossPrice
    AdditionalRemarks2
End Enum

Class ShopItem

    Public Sub New()
        PrintTypeSetting = PrintType.Detailed
    End Sub

    Public Sub New(ByVal StringArray() As String)
        Title = StringArray(FieldOrder.Titel)
        GrossPrice = Double.Parse(StringArray(FieldOrder.GrossPrice)) / 100
        OrderNumber = StringArray(FieldOrder.OrderNumber)
        PrintTypeSetting = PrintType.Detailed
    End Sub

    Public Property Title() As String

    Public Property OrderNumber() As String

    Public Property NetPrice() As Double

    Public ReadOnly Property NetPriceFormatted() As String
        Get
            Return NetPrice.ToString("#,##0.00") + " $"
        End Get
    End Property

    Public Overridable Property GrossPrice() As Double
        Get
            Return NetPrice * 1.19
        End Get

        Set(ByVal Value As Double)
            NetPrice = Value / 1.19
        End Set
    End Property

    Public ReadOnly Property GrossPriceFormatted() As String
        Get
            Return GrossPrice.ToString("#,##0.00") + " $"
        End Get
    End Property

    Public ReadOnly Property VATAmountFormatted() As String
        Get
            Return (GrossPrice - NetPrice).ToString("#,##0.00") + " $"
        End Get
    End Property

    Public Overridable ReadOnly Property Description() As String
        Get
            Return OrderNumber & vbTab & Title
        End Get
    End Property

    Public Property PrintTypeSetting() As PrintType

    Public Overrides Function ToString() As String

        If PrintTypeSetting = PrintType.Brief Then
            'Brief display format:
            Return MyClass.Description & vbCr & vbLf & _
                Me.NetPriceFormatted & vbTab & Me.VATAmountFormatted & vbTab & _
                Me.GrossPriceFormatted & vbCr & vbLf
        Else
            'Verbatim display format:
            Return Me.Description & vbCr & vbLf & _
                Me.NetPriceFormatted & vbTab & Me.VATAmountFormatted & vbTab & _
                Me.GrossPriceFormatted & vbCr & vbLf
        End If
    End Function

End Class

Class BookItem
    Inherits ShopItem

    Public Sub New(ByVal StringArray() As String)
        MyBase.New(StringArray)
        Author = StringArray(FieldOrder.AdditionalRemarks1)
    End Sub

    Public Overridable Property Author() As String

    Public Overrides Property GrossPrice() As Double
        Get
            Return NetPrice * 1.07
        End Get

        Set(ByVal Value As Double)
            NetPrice = Value / 1.07
        End Set
    End Property

    Public Overrides ReadOnly Property Description() As String
        Get
            Return OrderNumber & vbTab & Title & vbCr & vbLf & "Author: " & Author
        End Get
    End Property

End Class

Class DVDItem
    Inherits ShopItem

    Private myRunningTime As Integer
    Private myActor As String

    Public Sub New(ByVal StringArray() As String)
        MyBase.New(StringArray)
        RunningTime = Integer.Parse(StringArray(FieldOrder.AdditionalRemarks1))
        Actor = StringArray(FieldOrder.AdditionalRemarks2)
    End Sub

    Public Overridable Property RunningTime() As Integer
        Get
            Return myRunningTime
        End Get
        Set(ByVal Value As Integer)
            myRunningTime = Value
        End Set
    End Property

    Public Overridable Property Actor() As String
        Get
            Return myActor
        End Get
        Set(ByVal Value As String)
            myActor = Value
        End Set
    End Property

    Public Overrides ReadOnly Property Description() As String
        Get
            Return OrderNumber & vbTab & Title & vbCr & vbLf & "Running time: " & myRunningTime & " Min." & vbCr & vbLf & "Main actor: " & Actor
        End Get
    End Property

End Class

Class DynamicList

    Private myStep As Integer = 4         ' Step size by which the array is increased.
    Private myCurrentArraySize As Integer ' Actual array size.
    Private myCurrentCounter As Integer   ' Pointer to current item.
    Private myArray() As ShopItem         ' Array with the shop items.

    Sub New()
        myCurrentArraySize = myStep
        ReDim myArray(myCurrentArraySize - 1)
    End Sub

    Sub Add(ByVal Item As ShopItem)

        'Store item in array.
        myArray(myCurrentCounter) = Item

        'Increase pointer to item in array.
        myCurrentCounter += 1

        'Check if the current array limit has been reached,
        If myCurrentCounter = myCurrentArraySize - 1 Then
            'Create new array with more memory and copy items to that new location.

            'To that end: Array becomes larger:
            myCurrentArraySize += myStep

            'Create temporary Array
            Dim locTempArray(myCurrentArraySize - 1) As ShopItem

            'Copy every element. With this method,
            'this performs extremely fast, because firstly,
            'only the pointers to the actual elements are being copied.
            'And Secondly, it is highly optimized native code.
            Array.Copy(myArray, locTempArray, myArray.Length)

            'Temp array becomes main array, and we're done!
            'The previous list of pointers in myArray which is now orphaned
            'gets disposed by the Garbage Collector.
            myArray = locTempArray
        End If
    End Sub

    'Returns the current number of items in this list.
    Public Overridable ReadOnly Property Count() As Integer
        Get
            Return myCurrentCounter
        End Get
    End Property

    'Allows re-assignment of an item's value.
    Default Public Overridable Property Item(ByVal Index As Integer) As ShopItem
        Get
            Return myArray(Index)
        End Get

        Set(ByVal Value As ShopItem)
            myArray(Index) = Value
        End Set
    End Property
End Class
