﻿Imports System.ComponentModel
Imports System.Runtime.InteropServices

Public Class HighSpeedTimer

    Private myTimerID As Integer                ' Multimedia-timer ID
    Private myUser As Integer                   ' Custimized parameter - not used.
    Private myTimerCapabilities As TimerCaps    ' Contains the TimeCaps that describe

    ' the timer resolution
    Private myMode As Integer = 1               ' 0=once, 1=periodically. 1 is pre-defined.
    Private myPeriodInMs As Integer             ' How often to trigger?
    Private myresolutionInMs As Integer         ' Timer resolution?
    Private myHasStarted As Boolean             ' Is the timer running?

    'Delegate necessary for the call-back from the operating system,
    'when the timer has run out...
    Private Delegate Sub TimeProc(ByVal id As Integer, ByVal msg As Integer,
                                    ByVal user As Integer, ByVal reserved1 As Integer,
                                    ByVal reserved2 As Integer)

    '...and is defined in this delegate variable.
    Private myCallBackTimeProc As TimeProc

    'The event triggered by the call-back routine,
    'when the timer has run out.
    Public Event Elapsed(ByVal sender As Object, ByVal e As EventArgs)

    'Determines the capabilities of the timer on this computer
    <DllImport("winmm.dll")>
    Private Shared Function timeGetDevCaps(ByRef caps As TimerCaps,
                                           ByVal sizeOfTimerCaps As Integer) As Integer

    End Function

    'Creates and starts the multimedia-timer.
    <DllImport("winmm.dll")>
    Private Shared Function timeSetEvent(ByVal delay As Integer,
                                        ByVal resolution As Integer,
                                        ByVal CallBackProc As TimeProc, ByVal user As Integer,
                                        ByVal mode As Integer) As Integer
    End Function

    'Deletes a timer that is currently running.
    <DllImport("winmm.dll")>
    Private Shared Function timeKillEvent(ByVal id As Integer) As Integer
    End Function

    ''' <summary>
    ''' Creates an instance of this class and sets the repetition frequency
    ''' of the timer trigger and the resolution in milliseconds.
    ''' </summary>
    ''' <param name="periodInMs">Repetition frequency – how often should
    ''' the timer trigger.</param>
    ''' <param name="resolutionInMs">Timer resolution (the value 1ms delivers
    ''' the most correct result).</param>
    ''' <remarks></remarks>
    Sub New(ByVal periodInMs As Integer, ByVal resolutionInMs As Integer)

        'Determines the capabilities of the multimedia-timer which can
        'then be called via TimerCapabilities.
        Dim ret = timeGetDevCaps(myTimerCapabilities,
        Marshal.SizeOf(myTimerCapabilities))
        myPeriodInMs = periodInMs
        myresolutionInMs = resolutionInMs
        myCallBackTimeProc = AddressOf CallBackTimeProc
    End Sub

''' <summary>
    ''' Starts the timer.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Start()

        If myHasStarted Then
            Return
        End If

        Dim ret = timeSetEvent(myPeriodInMs, myresolutionInMs,
        myCallBackTimeProc, myUser, myMode)

        Dim hr = Marshal.GetHRForLastWin32Error
        If ret = 0 Then
            Throw New Win32Exception("The timer could not be started!")
        Else
            myTimerID = ret
            myHasStarted = True
        End If
    End Sub

    ''' <summary>
    ''' Stops the timer.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub [Stop]()

        If Not myHasStarted Then
            Return
        End If

        timeKillEvent(myTimerID)
    End Sub

    'The call-back routine which is called by the operating system
    'when the timer has run out.
    Private Sub CallBackTimeProc(ByVal id As Integer, ByVal msg As Integer,
                                    ByVal user As Integer, ByVal param1 As Integer,
                                    ByVal param2 As Integer)

        RaiseEvent Elapsed(Me, EventArgs.Empty)
    End Sub


    ''' <summary>
    ''' Determines the possibilities the system has regarding the timer.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimerCapabilities As TimerCaps
        Get
            Return myTimerCapabilities
        End Get
    End Property

    ''' <summary>
    ''' Determines whether the timer has been started.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property HasStarted As Boolean
        Get
            Return myHasStarted
        End Get
    End Property

End Class

'Being used for passing arguments to platform invoke calls to the operationg system.
<StructLayout(LayoutKind.Sequential)>
Public Structure TimerCaps
    Public PeriodMin As Integer
    Public periodMax As Integer
End Structure

