Public Class frmMain

    Private WithEvents myTimer As Timer
    Private WithEvents myAppointmentList As AppointmentList

    'The background color of the clock
    'which changes with a continuous alarm.
    Private myCurrentColor As Color

    'Alarm duration in 500ms steps (=25 seconds).
    Private myAlarmDuration As Integer = 50

    'Counter for the duration of the alarm remainder.
    Private myAlarmDownCounter As Integer

    'True: Alarm is currently active --> the clock blinks.
    Private myAlarmStatus As Boolean

    'If this string is Nothing, a text message is displayed
    'on the clock face, otherwise nothing.
    Private myLastAlarmMessage As String

    Sub New()
        'This call is required for the Windows Form Designer.
        InitializeComponent()

        'This timer is required for displaying the clock
        'and making it blink if the alarm continues.
        myTimer = New Timer()
        myTimer.Interval = 500
        myTimer.Start()

        'The default background color of the clock is white.
        myCurrentColor = Color.White

        'AppointmentList is still empty.
        myAppointmentList = New AppointmentList
    End Sub

    'Event handler method which is being raised when
    'the alarm trigger signales because a certain time has been reached.
    Private Sub myAlarmTrigger_Alarm(ByVal Sender As Object, ByVal e As AlarmEventArgs) Handles myAppointmentList.Alarm

        'Alarm clock rings!
        myAlarmStatus = True

        'It rings this long:
        myAlarmDownCounter = myAlarmDuration

        'We must be able to turn off the ringing.
        btnTurnOffAlarm.Enabled = True

        'Set message text
        myLastAlarmMessage = e.AlarmText

        'Remove from list
        lstAppointments.Items.Remove(Sender)

        'Remove from appointment list
        myAppointmentList.Remove(DirectCast(Sender, AlarmTrigger))
    End Sub

    'Event handling method which is being raised,
    'when the content of the picturebox has to be repainted.
    Private Sub picWecker_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) _
                    Handles picAlarmClock.Paint

        If myAppointmentList IsNot Nothing AndAlso myAppointmentList.Count > 0 Then
            'If there are appointments, then draw the tag for the next one.
            DrawClock(e.Graphics, Date.Now, myAppointmentList.NextAppointment, _
                      myLastAlarmMessage, myCurrentColor)
        Else
            DrawClock(e.Graphics, Date.Now, myLastAlarmMessage, myCurrentColor)
        End If
    End Sub

    'Being called, when the user as resized the window in any way.
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        'Don't forget to call the original method, when overwriting
        'event raising methods!!
        MyBase.OnResize(e)

        'Paint the content of the picturebox, if the size has changed.
        picAlarmClock.Invalidate()

    End Sub

    'Event handling method is triggered
    'when the timer fires. This happens
    'every 500 milliseconds, at which point we redraw
    'the complete clocktaking background blinking
    'into consideration if the alarm clock is going off.
    Private Sub myTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles myTimer.Tick

        'Is the alarm going off?
        If myAlarmStatus Then

            'Yes, change colors every 500 ms
            If myCurrentColor = Color.White Then
                myCurrentColor = Color.Red
            Else
                myCurrentColor = Color.White
            End If

            'Reduce alarm duration counter
            myAlarmDownCounter -= 1
            If myAlarmDownCounter = 0 Then
                'Switch off alarm when it has run out.
                SwitchOffAlarm(True)
            End If
        End If

        'Redraw entire clock in any case
        picAlarmClock.Invalidate()
    End Sub

    'Called on click on the add button.
    Private Sub btnAdd_Click(ByVal sender As System.Object, _
                    ByVal e As System.EventArgs) Handles btnAdd.Click

        Dim locAlarmTime As Date

        'Retrieve time and appointment reason from the TextBox control.
        If Date.TryParse(mtbAlarmTime.Text, locAlarmTime) And Not String.IsNullOrEmpty(txtReason.Text) Then

            'Instantiate new AlarmTrigger object,
            'which is then added directly to the ListBox...
            Dim locAlarm As New AlarmTrigger(locAlarmTime, txtReason.Text, True)
            lstAppointments.Items.Add(locAlarm)

            '...as well as to the appointment list.
            myAppointmentList.Add(locAlarm)

            'Redraw the clock, so that the ring time for the next appointment
            'is displayed as a red line in the clock face!
            picAlarmClock.Invalidate()
        Else

            'Oops! There is no such time - TryParse
            'has failed.
            MessageBox.Show("Please check your entry for errors!")
        End If

    End Sub

    'If there is a selected appointment,
    'the delete button can be enabled...
    Private Sub lstAppointments_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstAppointments.SelectedIndexChanged
        If lstAppointments.SelectedIndex > -1 Then
            btnDelete.Enabled = True
        Else
            '...otherwise not.
            btnDelete.Enabled = False
        End If
    End Sub

    'Switches of the alarm.
    Private Sub SwitchOffAlarm(ByVal reactivate As Boolean)

        'Reset status of alarm.
        myAlarmStatus = False

        'Deactivate "Snooze" Button
        btnTurnOffAlarm.Enabled = False

        'Background is reset to white.
        myCurrentColor = Color.White

        'Reset the alarm message.
        myLastAlarmMessage = Nothing

    End Sub

    '"Snooze" has been pressed...
    Private Sub btnTurnOffAlarm_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTurnOffAlarm.Click
        'so switch of the alarm, but (true) reschedule the 
        'alarm time for the next day.
        SwitchOffAlarm(True)
    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        Me.Close()
    End Sub

#Region "Code part, which draws the clock on the surface."
    Private Sub DrawClock(ByVal g As Graphics, ByVal Time As Date, ByVal MessageText As String, ByVal Background As Color)
        DrawClock(g, Time, Nothing, MessageText, Background)
    End Sub

    Private Sub DrawClock(ByVal g As Graphics, ByVal Time As Date, ByVal selectionTime As Nullable(Of Date), _
                    ByVal MessageText As String, ByVal background As Color)

        'Create Brushes and Pens
        Dim locPenWidth As Single = 3
        Dim locHalfPenWidth As Single = locPenWidth / 2
        Dim locCenter As PointF
        Dim locSize As SizeF

        Dim locClockDimension As RectangleF = New RectangleF( _
            locHalfPenWidth, locHalfPenWidth, _
            (picAlarmClock.ClientSize.Width - 1) - locPenWidth, _
            (picAlarmClock.ClientSize.Height - 1) - locPenWidth)

        'Find the center
        locCenter.X = picAlarmClock.ClientSize.Width / 2.0F
        locCenter.Y = picAlarmClock.ClientSize.Height / 2.0F

        'Size without the clock's border
        locSize.Width = locClockDimension.Width / 2 - 5
        locSize.Height = locClockDimension.Height / 2 - 5

        'Enable AntiAlias
        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        g.Clear(background)

        'Paint outer circle of the clock.
        g.DrawEllipse(New Pen(Color.Black, locPenWidth), locClockDimension)

        'Paint the clock's face.
        Dim locOffset As Single
        For locAngle As Single = 0 To 359 Step 30
            If locAngle = 0 Or locAngle = 90 Or locAngle = 180 Or locAngle = 270 Then
                locOffset = 20
            Else
                locOffset = 10
            End If

            Dim locPcStart As New PolarCoordinate(locCenter, locSize.Width - 5, locSize.Height - 5, locAngle)
            Dim locPcEnd As New PolarCoordinate(locCenter, locSize.Width - locOffset, _
                                locSize.Height - locOffset, locAngle)
            g.DrawLine(Pens.Black, locPcStart.Cartesian.X, _
                                  locPcStart.Cartesian.Y, _
                                  locPcEnd.Cartesian.X, _
                                  locPcEnd.Cartesian.Y)
        Next

        'Paint the clock's hands

        'Parameter for hour hand
        Dim locHourHandAngle As Single = Time.Hour
        If locHourHandAngle > 12 Then
            locHourHandAngle -= 12
        End If
        locHourHandAngle *= 30.0F
        locHourHandAngle += (Time.Minute / 2.0F)
        locHourHandAngle -= 90


        'Parameter for the alarm tag
        Dim locAlarmTagAngle As Single
        If selectionTime.HasValue Then
            locAlarmTagAngle = selectionTime.Value.Hour
            If locAlarmTagAngle > 12 Then
                locAlarmTagAngle -= 12
            End If
            locAlarmTagAngle *= 30.0F
            locAlarmTagAngle += (selectionTime.Value.Minute / 2.0F)
            locAlarmTagAngle -= 90
        End If

        'Parameter for the minute hand
        Dim locMinuteHandAngle As Single = Time.Minute
        locMinuteHandAngle *= 6
        locMinuteHandAngle -= 90

        'Parameter for the second hand
        Dim locSecondHandAngle As Single = Time.Second
        locSecondHandAngle *= 6
        locSecondHandAngle -= 90

        'First draw the hour hand...
        Dim locHandStart As New PolarCoordinate(locCenter, 0, 0, locHourHandAngle)
        Dim locHandEnd As New PolarCoordinate(locCenter, locSize.Width / 2, locSize.Height / 2, locHourHandAngle)
        g.DrawLine(New Pen(Color.Blue, 5), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... then the minute hand ...
        locHandStart = New PolarCoordinate(locCenter, 0, 0, locMinuteHandAngle)
        locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 25, locSize.Height - 25, locMinuteHandAngle)
        g.DrawLine(New Pen(Color.Blue, 3), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... then the second hand ...
        locHandStart = New PolarCoordinate(locCenter, 0, 0, locSecondHandAngle)
        locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 20, locSize.Height - 20, locSecondHandAngle)
        g.DrawLine(New Pen(Color.Black, 1), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... and if there is an alarm time to tag, draw that as well!
        If selectionTime.HasValue Then
            locHandStart = New PolarCoordinate(locCenter, locSize.Width / 4 * 3, locSize.Height / 4 * 3, locAlarmTagAngle)
            locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 5, locSize.Height - 5, locAlarmTagAngle)
            g.DrawLine(New Pen(Color.Red, 5), locHandStart.Cartesian.X, _
                                               locHandStart.Cartesian.Y, _
                                               locHandEnd.Cartesian.X, _
                                               locHandEnd.Cartesian.Y)

        End If

        'New in this version: the MessageText
        If Not String.IsNullOrEmpty(MessageText) Then
            lblAlarmMessage.Text = MessageText
            lblAlarmMessage.Visible = True
        Else
            lblAlarmMessage.Visible = False
        End If
    End Sub
#End Region

End Class
