Module Arrays

    Sub Main()

        ' This project combines several examples.
        ' Just change the following method call (to Example2, Example3, etc.)
        ' for running the other examples.
        Example1()

    End Sub

    Sub Example1()
        'Declare an array with 10 elements.
        'Important: Unlike in C# or C++ you specify the index of the
        'last element, not the number of elements!
        'Element count starts at 0.
        'Therefore the following statement defines 10 elements:
        Dim locIntArray(9) As Integer

        'Initialize random generator.
        Dim locRandom As New Random(Now.Millisecond)

        For count As Integer = 0 To 9
            locIntArray(count) = locRandom.Next
        Next

        For count As Integer = 0 To 9
            Console.WriteLine("Element No. {0} has the value {1}", count,
            locIntArray(count))
        Next

        Console.ReadLine()
    End Sub

    Sub Example2()

        Dim locStringsCount As Integer = 15
        Dim locStringArray(locStringsCount) As String
        locStringArray = GenerateStrings(locStringsCount, 30)
        PrintStrings(locStringArray)
        Console.ReadLine()
    End Sub

    Sub Example3()

        Dim locNumberStrings As Integer = 15
        Dim locStringArray As String()
        locStringArray = GenerateStrings(locNumberStrings, 30)
        Console.WriteLine("Initial size: {0} elements. Content follows:",
                            locStringArray.Length)

        Console.WriteLine(New String("="c, 40))
        PrintStrings(locStringArray)

        'We need 10 more, but the old ones should remain!
        ReDim Preserve locStringArray(locStringArray.Length + 9)

        'Do the old ones really remain?
        Console.WriteLine()
        Console.WriteLine("Content check:", locStringArray.Length)
        Console.WriteLine(New String("="c, 40))
        PrintStrings(locStringArray)

        'Generate 10 further elements.
        Dim locTempStrings(9) As String

        '10 chars more per element, so we can recognize the new ones easily.
        locTempStrings = GenerateStrings(10, 40)

        'Copy into the "old" array, but only from index 15 onward.
        locTempStrings.CopyTo(locStringArray, 15)

        'And check what's really written inside!
        Console.WriteLine()
        Console.WriteLine("Content check:", locStringArray.Length)
        Console.WriteLine(New String("="c, 40))
        PrintStrings(locStringArray)
        Console.ReadLine()
    End Sub

    Function GenerateStrings(ByVal StringsCount As Integer, ByVal StringsLength As Integer) As String()

        Dim locRandom As New Random(Now.Millisecond)
        Dim locChars(StringsLength - 1) As Char
        Dim locStrings(StringsCount - 1) As String

        For locOutCount As Integer = 0 To StringsCount - 1
            For locInCount As Integer = 0 To StringsLength - 1
                Dim locIntTemp As Integer = Convert.ToInt32(locRandom.NextDouble * 52)
                If locIntTemp > 26 Then
                    locIntTemp += 97 - 26
                Else
                    locIntTemp += 65
                End If
                locChars(locInCount) = Convert.ToChar(locIntTemp)
            Next
            locStrings(locOutCount) = New String(locChars)
        Next

        Return locStrings

    End Function

    Sub PrintStrings(ByVal locStringArray As String())
        For count As Integer = 0 To locStringArray.Length - 1
            If Not (locStringArray(count) Is Nothing) Then
                Console.WriteLine(locStringArray(count))
            Else
                Console.WriteLine("--- NOTHING ---")
            End If
        Next
    End Sub

    Sub Example4()

        Dim locDoubles(100) As Double
        Dim locIntegers(100) As Integer
        Dim locDecimals(100) As Decimal
        Dim locRandom As New Random(Now.Millisecond)

        'Fill every array with 101 random numbers.
        '(remember: 100 is the upper bound, it's not the item count!)
        For count As Integer = 0 To 100
            locDoubles(count) = locRandom.NextDouble * locRandom.Next
            locDecimals(count) = Convert.ToDecimal(locRandom.NextDouble * locRandom.Next)
            locIntegers(count) = locRandom.Next
        Next

        Console.WriteLine("The greatest item of the Double arrays was {0}", CDbl(Max(locDoubles)))
        Console.WriteLine("The greatest item of the Integer arrays was {0}", CInt(Max(locIntegers)))
        Console.WriteLine("The greatest item of the Decimal arrays was {0}", CDec(Max(locDecimals)))
        Console.ReadLine()
    End Sub

    Enum Compare
        SmallerThan = -1
        EqualTo = 0
        GreaterThan = 1
    End Enum

    Function Max(ByVal Array As System.Array) As IComparable

        'It's empty, then quit.
        If Array.Length = 0 Then
            Return Nothing
        End If

        'Only comparable items can be compared,
        'but fortunately, all primitive data types are comparable,
        'since they're implementing IComparable.
        Dim locICItem As IComparable

        'Get first item as the base
        locICItem = DirectCast(Array.GetValue(0), IComparable)
        If locICItem Is Nothing Then
            'Doesn't implement IComparable, then we're done.
            Return Nothing
        End If

        For Each locICLoopItem As IComparable In Array
            If locICLoopItem Is Nothing Then
                Return Nothing
            End If

            If locICLoopItem.CompareTo(locICItem) = Compare.GreaterThan Then
                locICItem = locICLoopItem
            End If
        Next
        Return locICItem

    End Function

Sub Example5()

        'Declaration and definition of elements with Double values
        Dim locDoubleArray As Double() = {123.45F, 5435.45F, 3.14159274F}

        'Declaration and subsequent definition of elements with Integer values
        Dim locIntegerArray As Integer()
        locIntegerArray = New Integer() {1I, 2I, 3I, 3I, 4I}

        'Declaration and subsequent definition of elements with Date values
        Dim locDateArray As Date()
        locDateArray = New Date() {#12/24/2005#, #12/31/2005#, #3/31/2006#}

        'Declaration and definition of elements in the Char array:
        Dim locCharArray As Char() = {"V"c, "B"c, "."c, "N"c, "E"c, "T"c, " "c, _
                                        "r"c, "u"c, "l"c, "e"c, "s"c, "!"c}

        'Two-dimensional array
        Dim locTwoDimensional As Integer(,)
        locTwoDimensional = New Integer(,) {{10, 10}, {20, 20}, {30, 30}}

        'Jagged arrays (which is NOT 2 or more dimensional but arrays of arrays)
        Dim locJagged As Date()()
        locJagged = New Date()() {New Date() {#12/24/2004#, #12/31/2004#}, _
                                    New Date() {#12/24/2005#, #12/31/2005#}}

        Console.WriteLine("Parent array has {0} elements.", locJagged.Length)
        Console.WriteLine("Array of the 1. element has {0} elements.", locJagged(0).Length)
    End Sub

    Sub Example6()

        'Simple jagged; depth is not defined.
        Dim SimpleJagged(10)() As Integer

        'First element contains an Integer array with three elements.
        SimpleJagged(0) = New Integer() {10, 20, 30}

        'Second element contains an Integer array with eight elements.
        SimpleJagged(1) = New Integer() {10, 20, 30, 40, 50, 60, 70, 80}

        'Prints the third element of the second array element (30).
        Console.WriteLine(SimpleJagged(1)(2))

        'Assign everything in one go.
        SimpleJagged = New Integer()() {New Integer() {30, 20, 10}, _
                        New Integer() {80, 70, 60, 50, 40, 30, 20, 10}}

        'Prints the third element of the second array element (now 60).
        Console.WriteLine(SimpleJagged(1)(2))
        Console.ReadLine()

    End Sub

    Sub Example7()

        'Create Array
        Dim locNames As String() = {"Adriana", "Martina", "Hanna", "Gaby", "Michaela", "Miriam", "Ute", _
                                    "Leonie-Gundula", "Mellanie", "Uwe", "Andrea", "Klaus", "Anja", _
                                    "Myriam", "Daja", "Thomas", "Jos", "Kricke", "Flori", "Katrin", "Momo", _
                                    "Gareth", "Anne"}
        System.Array.Sort(locNames)
        PrintStrings(locNames)
        Console.ReadLine()

        Console.WriteLine()
        Console.WriteLine("Sort descending:")
        Array.Reverse(locNames)
        PrintStrings(locNames)
        Console.ReadLine()

    End Sub

    Sub Example8()

        'Create array:
        Dim locNames As String() = {"Adriana", "Martina", "Hanna", "Gaby", "Michaela", "Miriam", "Ute", _
                                    "Leonie-Gundula", "Mellanie", "Uwe", "Andrea", "Klaus", "Anja", _
                                    "Myriam", "Daja", "Thomas", "Jos", "Kricke", "Flori", "Katrin", "Momo", _
                                    "Gareth", "Anne", "Jrgen", "Gaby"}
        System.Array.Sort(locNames)
        Console.WriteLine("Jrgen was found at position {0}", _
                System.Array.BinarySearch(locNames, "Jrgen"))
        Console.ReadLine()
    End Sub

End Module
