Imports System.Collections.ObjectModel

Public Class frmListForm

    Private myOrderBy As SortAddressesBy                ' Current sort order
    Private myAddresses As List(Of Address)             ' All addresses

    Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
        MyBase.OnLoad(e)

        'Generate random addresses.
        myAddresses = Address.RandomAddresses(50)

        'Set up listview columns.
        Preperations()

        'Show the generated random addresses.
        ShowItems()
    End Sub

    'Sets up the list view.
    Sub Preperations()
        With Me.lvwAdressen

            'Clear all Items in the list view.
            .Items.Clear()
            With .Columns

                'Clear the column headers.
                .Clear()

                'Add column headers.
                .Add("Last name", -2)
                .Add("First name", -2)
                .Add("ZIP", -2)
                .Add("City", -2)
            End With
        End With
    End Sub

    Sub ShowItems()

        'Supresses the Paint events until EndUpdate. This way,
        'the list doesn't get refreshed after every insert of
        'an item, hence the rebuild of the list is much faster,
        'and it doesn't flicker.
        Me.lvwAdressen.BeginUpdate()

        'Clear all items in the list view.
        Me.lvwAdressen.Items.Clear()

        'Call ItemInList for every item.
        myAddresses.ForEach(Sub(item)
                                Dim locLvwItem As New ListViewItem(item.LastName)

                                'Set sub items.
                                With locLvwItem.SubItems
                                    .Add(item.FirstName)
                                    .Add(item.ZIP)
                                    .Add(item.City)
                                End With

                                'To find the original item later, we put the reference to the
                                'address object it was created from in the list view item's tag.
                                locLvwItem.Tag = item

                                'And then we add it to the list view.
                                lvwAdressen.Items.Add(locLvwItem)

                            End Sub)

        'This is how we align the columns so every item will fit.
        For Each locCol As ColumnHeader In Me.lvwAdressen.Columns
            locCol.Width = -2
        Next

        'And, we're done - the paint event can be raised again.
        Me.lvwAdressen.EndUpdate()
    End Sub

    'Called when one of the columns has been clicked.
    Private Sub lvwAdressen_ColumnClick(ByVal sender As Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles lvwAdressen.ColumnClick

        'Convert column number of e.Column into SortAddressesBy
        myOrderBy = CType(e.Column, SortAddressesBy)

        Select Case myOrderBy
            Case SortAddressesBy.LastName
                myAddresses.Sort(Function(adr1, adr2) String.Compare(adr1.LastName, adr2.LastName))

            Case SortAddressesBy.FirstName
                myAddresses.Sort(Function(adr1, adr2) String.Compare(adr1.FirstName, adr2.FirstName))

            Case SortAddressesBy.ZIP
                myAddresses.Sort(Function(adr1, adr2) String.Compare(adr1.ZIP, adr2.ZIP))

            Case SortAddressesBy.City
                'Multiline lambda would work also ...
                myAddresses.Sort(Function(adr1, adr2)
                                     '... but in that case, don't forget the 'Return'!
                                     Return String.Compare(adr1.City, adr2.City)
                                 End Function)

        End Select

        'Display the resorted items.
        ShowItems()
    End Sub

    Private Sub btnSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSearch.Click

        'Query search text
        Dim searchForm As New frmSearch
        Dim currentSearchText As String    ' The most recently queried search text

        'Remember the search text, so the predicate delegate
        'can access it.
        currentSearchText = searchForm.searchText
        If String.IsNullOrEmpty(currentSearchText) Then
            Return
        End If

        'Here the search begins!
        Dim locGefAdr As Address = Nothing

        Select Case myOrderBy
            Case SortAddressesBy.LastName
                locGefAdr = myAddresses.Find(Function(adr) adr.LastName = currentSearchText)
            Case SortAddressesBy.FirstName
                locGefAdr = myAddresses.Find(Function(adr) adr.FirstName = currentSearchText)
            Case SortAddressesBy.ZIP
                locGefAdr = myAddresses.Find(Function(adr) adr.ZIP = currentSearchText)
            Case SortAddressesBy.City
                'Mehrzeilig ginge natrlich auch hier wieder ...
                locGefAdr = myAddresses.Find(Function(adr)
                                                 '...dann aber - wieder - 'Return' nicht vergessen!
                                                 Return adr.City = currentSearchText
                                             End Function)
        End Select

        'When an item was found, select it.
        If locGefAdr IsNot Nothing Then

            'Search all ListView items, and check wether ...
            For Each locLvwItem As ListViewItem In Me.lvwAdressen.Items

                '... the Tag reference corresponds to the reference of the searched object.
                If locLvwItem.Tag Is locGefAdr Then

                    'Found! Select ListView item...
                    locLvwItem.Selected = True

                    '...and make sure that this is located in the visible range.
                    locLvwItem.EnsureVisible()
                    Return
                End If
            Next
        End If
    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        Me.Close()
    End Sub
End Class

Public Enum SortAddressesBy
    LastName
    FirstName
    ZIP
    City
End Enum
