Imports System.Collections.ObjectModel

Public Class frmMain

    Private myAddresses As Addresses         ' Alle Addresses
    Private myOrderBy As SortAddressesBy     ' Current sort order
    Private myCurrentSearchText As String    ' Last search text.

    Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
        MyBase.OnLoad(e)

        'Instanciate.
        myAddresses = New Addresses

        'Set up listview columns.
        Preperations()

        'Show the generated random addresses.
        ShowItems()
    End Sub

    'Sets up the list view.
    Sub Preperations()
        With Me.lvwAddresses

            'Clear all Items in the list view.
            .Items.Clear()
            With .Columns

                'Clear the column headers.
                .Clear()

                'Add column headers.
                .Add(My.Resources.Column_LastName, -2)
                .Add(My.Resources.Column_FirstName, -2)
                .Add(My.Resources.Column_Street, -2)
                .Add(My.Resources.Column_ZIP, -2)
                .Add(My.Resources.Column_City, -2)
            End With
        End With
    End Sub

    Sub ShowItems()

        'Supresses the Paint events until EndUpdate. This way,
        'the list doesn't get refreshed after every insert of
        'an item, hence the rebuild of the list is much faster,
        'and it doesn't flicker.
        Me.lvwAddresses.BeginUpdate()

        'Clear all items in the list view.
        Me.lvwAddresses.Items.Clear()

        'Call ItemInList for every item.
        myAddresses.ForEach(Sub(Item As Address)

                                'New ListVierw Item - Last name comes first.
                                Dim locLvwItem As New ListViewItem(Item.LastName)

                                'Set up sub items.
                                With locLvwItem.SubItems
                                    .Add(Item.FirstName)
                                    .Add(Item.Street)
                                    .Add(Item.Zip)
                                    .Add(Item.City)
                                End With

                                'To find the original item later, we put the reference to the
                                'address object it was created from in the list view item's tag.
                                locLvwItem.Tag = Item

                                'And then we add it to the list view.
                                lvwAddresses.Items.Add(locLvwItem)
                            End Sub)

        'This is how we align the columns so every item will fit.
        For Each locCol As ColumnHeader In Me.lvwAddresses.Columns
            locCol.Width = -2
        Next

        'And, we're done - the paint event can be raised again.
        Me.lvwAddresses.EndUpdate()
    End Sub

    'Called when one of the columns has been clicked.
    Private Sub lvwAddresses_ColumnClick(ByVal sender As Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles lvwAddresses.ColumnClick

        'Convert column number of e.Column into SortAddressesBy
        myOrderBy = CType(e.Column, SortAddressesBy)

        myAddresses.Sort(Function(x As Address, y As Address) As Integer

                             Select Case myOrderBy
                                 Case SortAddressesBy.LastName
                                     Return x.LastName.CompareTo(y.LastName)
                                 Case SortAddressesBy.Town
                                     Return x.City.CompareTo(y.City)
                                 Case SortAddressesBy.Zip
                                     Return x.Zip.CompareTo(y.Zip)
                                 Case SortAddressesBy.Street
                                     Return x.Street.CompareTo(y.Street)
                                 Case Else
                                     Return x.FirstName.CompareTo(y.FirstName)
                             End Select
                         End Function)

        'Display the resorted items.
        ShowItems()
    End Sub

    Private Sub tsmSearchAddress_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmSearchAddress.Click

        'Query search text
        Dim searchForm As New frmSearch

        'Remember the search text, so the predicate delegate
        'can access it.
        myCurrentSearchText = searchForm.SearchText
        If String.IsNullOrEmpty(myCurrentSearchText) Then
            Return
        End If

        'Here the search begins!
        Dim locGefundeneAdresse As Address = _
           myAddresses.Find(Function(adr As Address) As Boolean
                                'Where always searching for the items of the last selected/clicked column.
                                Select Case myOrderBy
                                    Case SortAddressesBy.LastName
                                        Return adr.LastName = myCurrentSearchText
                                    Case SortAddressesBy.Town
                                        Return adr.City = myCurrentSearchText
                                    Case SortAddressesBy.Zip
                                        Return adr.Zip = myCurrentSearchText
                                    Case SortAddressesBy.Street
                                        Return adr.Street = myCurrentSearchText
                                    Case Else
                                        Return adr.FirstName = myCurrentSearchText
                                End Select
                            End Function)

        'When an item was found, select it.
        If locGefundeneAdresse IsNot Nothing Then

            'Search all ListView items, and check wether ...
            For Each locLvwItem As ListViewItem In Me.lvwAddresses.Items

                '... the Tag reference corresponds to the reference of the searched object.
                If locLvwItem.Tag Is locGefundeneAdresse Then

                    'Found! Select ListView item...
                    locLvwItem.Selected = True

                    '...and make sure that this is located in the visible range.
                    locLvwItem.EnsureVisible()
                    Return
                End If
            Next
        End If
    End Sub

    Private Sub tsmDeleteAddresses_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmDeleteAddresses.Click
        If lvwAddresses.SelectedItems IsNot Nothing AndAlso lvwAddresses.SelectedItems.Count > 0 Then

            Dim locDr As DialogResult = MessageBox.Show(My.Resources.MB_Reqest_Delete_Body, _
                                My.Resources.MB_Request_Delete_Titel, MessageBoxButtons.YesNo, _
                                MessageBoxIcon.Question)
            If locDr = Windows.Forms.DialogResult.Yes Then
                For Each lvwItem As ListViewItem In lvwAddresses.SelectedItems
                    myAddresses.Remove(DirectCast(lvwItem.Tag, Address))
                Next
                ShowItems()
            End If
        End If
    End Sub

    Private Sub tsmAddRandomAddresses_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmAddRandomAddresses.Click

        For Each locAdresse As Address In Addresses.RandomAddresses(20)
            myAddresses.Add(locAdresse)
        Next
        ShowItems()
    End Sub

    'Called, when the user clicks open from the file menu.
    Private Sub tsmLoadAddresses_Click(ByVal sender As System.Object, _
                        ByVal e As System.EventArgs) Handles tsmLoadAddresses.Click

        Dim fileOpenDialog As New OpenFileDialog
        With fileOpenDialog
            .CheckFileExists = True
            .CheckPathExists = True
            .DefaultExt = "*.xml"
            .Filter = "Adresso XML files" & " (" & "*.adrxml" & ")|" & "*.adrxml" & "|All files (*.*)|*.*"
            Dim dialogErgebnis As DialogResult = .ShowDialog
            If dialogErgebnis = Windows.Forms.DialogResult.Cancel Then
                Exit Sub
            End If

            'Deserialize addresses.
            myAddresses = Addresses.XmlDeserialize(.FileName)

            'Show the items.
            ShowItems()
        End With
    End Sub

    'Called, when the user click save from the file menu.
    Private Sub tsmAdresslisteSpeichern_Click(ByVal sender As System.Object, _
                        ByVal e As System.EventArgs) Handles tsmSaveAddresses.Click

        Dim fileSaveDialog As New SaveFileDialog

        With fileSaveDialog
            .CheckPathExists = True
            .DefaultExt = "*.xml"
            .Filter = "Addresso XML files" & " (" & "*.adrxml" & ")|" & "*.adrxml" & "|All files (*.*)|*.*"
            Dim dialRes As DialogResult = .ShowDialog
            If dialRes = Windows.Forms.DialogResult.Cancel Then
                Exit Sub
            End If

            'Serialize addresses.
            myAddresses.XMLSerialize(.FileName)
        End With
    End Sub
End Class

Public Enum SortAddressesBy
    LastName
    FirstName
    Street
    Zip
    Town
End Enum
