﻿Option Infer On

Module LinqDemo

    Private Const ELEMENTCOUNT = 50

    Private adrList As List(Of Contact) = Contact.RandomContacts(ELEMENTCOUNT)
    Private prdList As List(Of Product) = Product.RandomProducts(adrList)

    Sub Main()

        LinqQueryStructure()
        CombinedComplexQuery()
        SelectSpeedCompare()
        ConcatenatedLinqQueries()
        TypesOfConcatenatedLinqQueries()
        AsParallel()
        JoinDemo()
        CombiningCollections()
        GroupByDemo()
        GroupJoin()
        GroupByJoinedCombined()
        AggregateDemo()
        JoinAggregateDemo()
        PerformanceTest()
        Console.ReadKey()

    End Sub

    Sub LinqQueryStructure()

        Console.WriteLine("Result list 1:")
        Console.WriteLine("----------------")

        Dim resultList = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName Descending
                            Select ContaktId = adrElement.ID,
                                   adrElement.LastName,
                                   adrElement.FirstName,
                                   ZipCityCombination = adrElement.ZIP & " " & adrElement.City
                                    Where ContaktId > 50 And ContaktId < 100 And
                                           LastName Like "L*"

        For Each ergebnisItem In resultList
            With ergebnisItem
                Console.WriteLine(.ContaktId & ": " &
                                  .LastName & ", " & .FirstName &
                                  " - " & .ZipCityCombination)
            End With
        Next

        Console.WriteLine()
        Console.WriteLine("Result list 2:")
        Console.WriteLine("----------------")


        Dim resultList2 = From adrElement In adrList
                    Where adrElement.ID > 50 And adrElement.ID < 100 And
                            adrElement.LastName Like "L*"
                    Select KontaktId = adrElement.ID,
                           adrElement.LastName,
                           adrElement.FirstName,
                           ZipCityComb = adrElement.ZIP & " " & adrElement.City
                    Order By LastName, FirstName

        For Each ergebnisItem In resultList2
            With ergebnisItem
                Console.WriteLine(.KontaktId & ": " &
                                  .LastName & ", " & .FirstName &
                                  " - " & .ZipCityComb)
            End With
        Next
    End Sub

    Sub SelectSpeedCompare()

        Dim sw = Stopwatch.StartNew
        Console.WriteLine("Starting Test")
        sw.Start()
        Dim resultList = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName
                            Select ContactId = adrElement.ID,
                                   adrElement.LastName,
                                   adrElement.FirstName,
                                   ZipCityComb = adrElement.ZIP & " " & adrElement.City

        sw.Stop()
        Dim firstCount = resultList.Count
        Dim duration1 = sw.ElapsedMilliseconds

        sw.Reset()
        sw.Start()
        Dim resultList2 = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName

        Dim secondCount = resultList2.Count
        sw.Stop()
        Dim duration2 = sw.ElapsedMilliseconds

        Console.WriteLine("Duration of query with Select: " & duration1 & " for " & firstCount & " items in the result set.")
        Console.WriteLine("Duration of query without Select: " & duration2 & " for " & secondCount & " items in the result set.")
    End Sub

    Sub PerformanceTest()

        Console.WriteLine("Running query:")
        Console.WriteLine("--------------------")

        Dim sw = Stopwatch.StartNew
        Dim ergebnisListe = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName
                            Where adrElement.ID > 50 And adrElement.ID < 100 And
                                    adrElement.LastName Like "L*"

        Dim liste = ergebnisListe.ToList
        sw.Stop()
        Console.WriteLine("The query required: " & sw.ElapsedMilliseconds.ToString & " milliseconds.")
    End Sub

    Sub CombinedComplexQuery()

        Dim adrGroupedList = From adrItem In adrList
                        Join productItem In prdList
                        On adrItem.ID Equals productItem.IDPurchasedBy
                        Select adrItem.ID, adrItem.LastName,
                               adrItem.FirstName, adrItem.ZIP,
                               productItem.ProductNo, productItem.ProductName,
                               productItem.Amount, productItem.UnitPrice,
                               ItemPrice = productItem.Amount * productItem.UnitPrice
                        Order By LastName, ProductNo
                        Where (ZIP > "0" And ZIP < "50000")
                        Group ProductNo, ProductName,
                              Amount, UnitPrice, ItemPrice
                        By ID, LastName, FirstName
                        Into Artikelliste = Group, ProductCount = Count(),
                             TotalPrice = Sum(ItemPrice)
                        Where TotalPrice > 1000


        For Each CustomerItem In adrGroupedList
            With CustomerItem
                Console.WriteLine(.ID & ": " & .LastName & ", " & .FirstName & " - " & .ProductCount & " Article, for a total of " & .TotalPrice & " $")
                Console.WriteLine("     Details:")
                For Each ArtItem In CustomerItem.Artikelliste
                    With ArtItem
                        Console.WriteLine("     " & .ProductNo & ": " & .ProductName &
                                          "(" & .Amount & " piece for a total of " &
                                          (.UnitPrice * .Amount).ToString("#,##0.00") & " $)")
                    End With
                Next
            End With

        Next
        Console.ReadKey()
    End Sub

    Sub ConcatenatedLinqQueries()

        Dim sw = Stopwatch.StartNew
        Console.WriteLine("Starting Test")
        sw.Start()
        Dim resultList = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName

        Dim resultList2 = From adrElement In resultList
                            Where adrElement.LastName Like "L*"

        resultList2 = From adrElement In resultList2
                            Where adrElement.ID > 50 And adrElement.ID < 200

        Dim firstCount = resultList2.Count
        sw.Stop()
        Console.WriteLine("Starting Test2")
        Dim duration1 = sw.ElapsedMilliseconds

        sw.Reset()
        sw.Start()

        Dim resultList3 = From adrElement In adrList
                            Order By adrElement.LastName, adrElement.FirstName
                            Where adrElement.LastName Like "L*" And
                            adrElement.ID > 50 And adrElement.ID < 200

        Dim secondCount = resultList3.Count
        'This repeats the actual execution of the query:
        secondCount = resultList3.Count
        sw.Stop()
        Dim duration2 = sw.ElapsedMilliseconds

        Console.WriteLine("Duration of concatenated query: " & duration1 & " for " & firstCount & " items in the result set.")
        Console.WriteLine("Duration of simple query: " & duration2 & " for " & secondCount & " items in the result set.")

        'Add an item which "would" match the query
        adrList.Add(New Contact(51, "Löffelmann", "Klaus", "Wiedenbrücker Straße 47", "59555", "Lippstadt"))
        Console.WriteLine("Items count of simple query: " & resultList2.Count)

    End Sub

    Sub TypesOfConcatenatedLinqQueries()

        Dim sw = Stopwatch.StartNew
        Console.WriteLine("Analysing types:" & vbNewLine)

        Dim resultList = From adrElement In adrList
        Dim resultListType = resultList.GetType
        Console.WriteLine("Although resultList is of type IEnumerable it actually is of type " & vbNewLine & resultListType.ToString & vbNewLine)

        resultList = From adrElement In resultList
                    Order By adrElement.LastName, adrElement.FirstName

        resultListType = resultList.GetType
        Console.WriteLine("Although resultList is of type IEnumerable it acually is of type " & vbNewLine & resultListType.ToString & vbNewLine)

        resultList = From adrElement In resultList
                     Where adrElement.LastName Like "L*"

        resultListType = resultList.GetType
        Console.WriteLine("Although resultList is of type IEnumerable it acually is of type " & vbNewLine & resultListType.ToString & vbNewLine)

        resultList = From adrElement In resultList
                     Where adrElement.ID > 50 And adrElement.ID < 200

        resultListType = resultList.GetType
        Console.WriteLine("Although resultList is of type IEnumerable it acually is of type " & vbNewLine & resultListType.ToString & vbNewLine)

        resultList = resultList.ToList
        resultListType = resultList.GetType
        Console.WriteLine("Although resultList is of type IEnumerable it acually is of type " & vbNewLine & resultListType.ToString & vbNewLine)

    End Sub

    Sub CombiningCollections()

        Dim resultList = From adrElement In adrList, artElement In prdList
                         Where adrElement.ID = artElement.IDPurchasedBy

        For Each item In resultList

            With (item)
                Console.WriteLine(.adrElement.ID & ": " & .adrElement.LastName &
                                  ", " & .adrElement.FirstName & ": " &
                                  .artElement.IDPurchasedBy & ": " & .artElement.ProductName)
            End With
        Next
    End Sub

    Sub JoinDemo()
        Dim resultList = From adrElement In adrList
                            Join artElement In prdList
                            On adrElement.ID Equals artElement.IDPurchasedBy

        For Each item In resultList
            With item
                Console.WriteLine(.adrElement.ID & ": " & .adrElement.LastName &
                                  ", " & .adrElement.FirstName & ": " &
                                  .artElement.IDPurchasedBy & ": " & .artElement.ProductName)
            End With
        Next
    End Sub

    Sub GroupByDemo()
        Dim resultList = From adrElement In adrList
                            Group By adrElement.LastName Into listOfContacts = Group
                            Order By LastName

        For Each item In resultList
            With item
                Console.WriteLine(item.LastName)
                For Each contact In item.listOfContacts
                    With contact
                        Console.WriteLine(.ID & ": " & .LastName & ", " & .FirstName)
                    End With
                Next
                Console.WriteLine()
            End With
        Next
    End Sub

    Sub GroupByJoinedCombined()
        Dim resultList = From adrElement In adrList
                            Join artElement In prdList
                            On adrElement.ID Equals artElement.IDPurchasedBy
                            Group artElement.IDPurchasedBy, ProductNo = artElement.ProductNo,
                                artElement.ProductName
                            By artElement.IDPurchasedBy, adrElement.LastName, adrElement.FirstName
                            Into listOfProducts = Group, ProductCount = Count() Order By LastName

        For Each contactItem In resultList
            With contactItem

                Console.WriteLine(.IDPurchasedBy & ": " & .LastName & ", " & .FirstName)
                For Each productItem In .listOfProducts
                    With productItem
                        Console.WriteLine("   " & .ProductNo & ": " & .ProductName)
                    End With
                Next
                Console.WriteLine()
            End With
        Next
    End Sub

    Sub GroupJoin()
        Dim itemList = From adrElement In adrList
                        Group Join artElement In prdList
                        On adrElement.ID Equals artElement.IDPurchasedBy Into listOfProducts = Group

        For Each contactItem In itemList
            With contactItem

                Console.WriteLine(.adrElement.ID & ": " &
                                  .adrElement.LastName & ", " &
                                  .adrElement.FirstName)
                For Each productItem In .listOfProducts
                    With productItem
                        Console.WriteLine("   " & .ProductNo & ": " & .ProductName)
                    End With
                Next
                Console.WriteLine()
            End With
        Next
    End Sub

    Sub AggregateDemo()

        Dim feignedSales = Aggregate artElement In prdList
                           Into TotalSum = Sum(artElement.UnitPrice)
        Console.WriteLine("Apparent total sales are {0:#,##0.00} $", feignedSales)

        Console.ReadKey()
        Console.Clear()

        Dim realSales = Aggregate artElement In prdList
                           Let productSales = artElement.UnitPrice * artElement.Amount
                           Into TotalSum = Sum(productSales)
        Console.WriteLine("Actual sales are {0:#,##0.00} $", realSales)
        Console.ReadKey()
        Console.Clear()

        Dim multipleInfo = Aggregate artElement In prdList
                           Let productSales = artElement.UnitPrice * artElement.Amount
                           Into TotalSum = Sum(productSales),
                                        priceMin = Min(artElement.UnitPrice),
                                        priceMax = Max(artElement.UnitPrice)

        Console.WriteLine("Total sales are {0:#,##0.00} $" & vbNewLine &
                          "Minimum and maximum prices are " &
                          "{1:#,##0.00} and {2:#,##0.00} $",
                          multipleInfo.TotalSum, multipleInfo.priceMin,
                         multipleInfo.priceMax)

    End Sub

    Sub JoinAggregateDemo()
        Dim resultList = From adrElement In adrList
                            Group Join artElement In prdList
                            On adrElement.ID Equals artElement.IDPurchasedBy
                            Into JoinedList = Group,
                            customerSales = Sum(artElement.UnitPrice * artElement.Amount)
                            Order By customerSales Descending
                            Take 10

        For Each item In resultList
            With item.adrElement
                Console.WriteLine(.ID & ": " & .LastName & ", " &
                                  .FirstName & " --- Sales:" &
                                  item.customerSales.ToString("#,##0.00") &
                                  " $")
            End With
        Next
    End Sub

    Sub SimpleQuery()

        Dim ergebnisListe = (From adrElememnt In adrList
                          Where adrElememnt.LastName > "L").ToList()
        If ergebnisListe IsNot Nothing Then
            ergebnisListe.ForEach(Sub(value)
                                      Console.WriteLine(value.ToString)
                                  End Sub)
        End If
    End Sub

    Sub AsParallel()

        Dim sw = Stopwatch.StartNew
        Console.WriteLine("Starting 1st query (as parallel)...")
        Dim resultList1 = From adrElement In adrList.AsParallel
                            Where adrElement.LastName Like "L*"
                            Order By adrElement.LastName, adrElement.FirstName

        Dim firstList = resultList1.ToList
        Dim lastElement = firstList(firstList.Count - 1)
        sw.Stop()
        Console.WriteLine("...finished 1st query.")
        Console.WriteLine()
        Dim duration1 = sw.ElapsedMilliseconds
        sw.Reset()

        'Rebuild the list
        adrList = Contact.RandomContacts(ELEMENTCOUNT)
        sw.Start()
        Console.WriteLine("Starting 2nd query (serial)...")

        Dim resultList2 = From adrElement In adrList
                            Where adrElement.LastName Like "L*"
                            Order By adrElement.LastName, adrElement.FirstName

        Dim secondList = resultList2.ToList
        sw.Stop()
        Console.WriteLine("...finished 1st query.")
        Console.WriteLine()
        Dim duration2 = sw.ElapsedMilliseconds

        Console.WriteLine("Duration of the parallalized query: " & duration1 & " for " & firstList.Count & " items in the result set.")
        Console.WriteLine("Duration of the serial query: " & duration2 & " for " & secondList.Count & " items in the result set.")
    End Sub

    Public Sub AnonymousClassDemo()

        Dim item = New With {.ItemName = "An Item",
                                    .ItemID = 42,
                                    .ItemsOtherProperty = "I'm a VB!!"}
    End Sub

End Module
