﻿Imports System.Data.Objects

Module EditData

    Sub main()

        ChangeData()
        InsertRelatedData()
        DeleteData()
        ChangeDataWithConcurrencyCheck()
    End Sub

    Sub ChangeData()

        Dim awContext As New AWEntities

        Dim vendor = (From supplierItem In awContext.Vendor _
                          Where supplierItem.Name.ToLower.StartsWith("hybrid") _
                          Order By supplierItem.Name).First

        'Change back and forth, so this example will work more than once:
        If vendor.Name = "Hybrid Bicycle Center" Then
            vendor.Name = "Hybrid Fahrrad Center"
        Else
            vendor.Name = "Hybrid Bicycle Center"
        End If

        awContext.SaveChanges()

        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadKey()

    End Sub

    Sub InsertRelatedData()

        Dim awContext As New AWEntities()

        Dim newSupplier As New Vendor With {.Name = "ActiveDevelop", .CreditRating = 1, _
                                       .AccountNumber = "ACTDEV00001", .ModifiedDate = Now}

        ' Deactivate the Trigger on the vendor table,
        ' since a vendor must not be deleted, actually.
        ' This said, we can make an exception in this case,
        ' since you are exclusively working on the database
        ' and most likely won't use replication (that's what the triggers 
        'are meant for)

        ' IMPORTANT: Note, that the trigger will be deactivated for all users,
        ' so you would risk inconsistent data in a real world scenario.
        ' We can do this on this playground - a real database administrator
        ' would become extremly angry, if we did this:
        awContext.ExecuteStoreCommand("disable trigger Purchasing.dVendor on Purchasing.Vendor")


        ' Let's check, if we have "leftovers" from previous experiments.
        ' If there are any then we'll delete them.
        ' To this end, we seek the data based on this account number (activedev.AccountNumber = newSupplier.AccountNumber)
        Dim existingRecords = (From activedev In awContext.Vendor
                                   Where activedev.AccountNumber = newSupplier.AccountNumber).ToList()

        If existingRecords.Count = 1 Then
            Dim actualExistingRecord = existingRecords(0)
            ' We have remainings,
            ' if we're not using LazyLoading, we reload the ProductVendors.
            If Not actualExistingRecord.ProductVendor.IsLoaded Then
                actualExistingRecord.ProductVendor.Load()
            End If
            Dim prodVendorCopy = actualExistingRecord.ProductVendor.ToArray()
            For Each prodvends In prodVendorCopy
                If Not prodvends.ProductReference.IsLoaded Then
                    prodvends.ProductReference.Load()
                End If
                awContext.DeleteObject(prodvends.Product)
                awContext.DeleteObject(prodvends)
            Next
            awContext.DeleteObject(actualExistingRecord)
            awContext.SaveChanges()
        End If

        'Create the new product
        Dim newProduct As New Product With {.Name = "Visual Basic 2010—Developer's Handbook", _
                                              .ProductNumber = "", _
                                              .ListPrice = 59, _
                                              .ModifiedDate = Now, _
                                              .DiscontinuedDate = #12/31/2011#, _
                                              .SellStartDate = #12/10/2008#, _
                                              .SellEndDate = #12/31/2011#, _
                                              .SafetyStockLevel = 5, _
                                              .ReorderPoint = 2}

        'Create new supplier product, connect with the product,
        'and connect with the supplier
        Dim newSupplierProduct = New ProductVendor With {.Product = newProduct, _
                                                .StandardPrice = 59, _
                                                .ModifiedDate = Now, _
                                                .UnitMeasureCode = "PC", _
                                                .AverageLeadTime = 19, _
                                                .MinOrderQty = 1, _
                                                .MaxOrderQty = 10}

        'Note: This appoach has deprecated
        'awContext.AddToVendor(newSupplier)
        'Use new approach
        awContext.Vendor.AddObject(newSupplier)
        newSupplier.ProductVendor.Add(newSupplierProduct)
        awContext.SaveChanges()

        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadKey()
    End Sub


    Sub DeleteData()

        Dim awContext As New AWEntities()

        'Get the entity object through the product:
        Dim supplierProduct = (From lieferantProduktItem In awContext.ProductVendor _
                                 From productItem In awContext.Product _
                                  Where productItem.ProductID = lieferantProduktItem.ProductID _
                                  Where productItem.Name = "Visual Basic 2010—Developer's Handbook" _
                                   Select lieferantProduktItem).First

        'Just to make sure:
        Console.WriteLine(supplierProduct.ProductID)

        'Load the parent entity objects.
        'First the supplier product which is assigned
        'to the determined product.
        supplierProduct.ProductReference.Load()

        'And we are loading the parent supplier just as easily.
        supplierProduct.VendorReference.Load()

        'We will temporarily have to remember the references
        'to the entities, because DeleteObject destroys them
        'with the parent table.
        Dim tmpProduct = supplierProduct.Product
        Dim tmpVendor = supplierProduct.Vendor

        'The deletion order is now ...
        awContext.DeleteObject(tmpVendor)

        '... no longer important.
        awContext.DeleteObject(tmpProduct)

        'Entity Framework establishes the correct order!
        awContext.SaveChanges()
        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadKey()
    End Sub

    Sub ChangeDataWithConcurrencyCheck()

        Dim awContext As New AWEntities()

        Dim supplier = (From supplierItem In awContext.Vendor _
                          Where supplierItem.Name.ToLower.StartsWith("hybrid") _
                          Order By supplierItem.Name).First

        'Change back and forth, so it works more than once:
        If supplier.Name = "Hybrid Bicycle Center" Then
            supplier.Name = "Hybrid Bicycle Center"
        Else
            supplier.Name = "Hybrid Cycle Center"
        End If
        'Set time stamp for the last change
        supplier.ModifiedDate = Date.Now

        Console.WriteLine("You can now change the data set with the VendorID {0} with SQL Server Management Studio.", supplier.VendorID)
        Console.WriteLine("Don’t forget to change at least one field and set its parallality concurrency mode to fixed (typically ModifiedDate).")
        Console.WriteLine("Or simply start the program again (e.g. with the Windows Explorer from the Bin directory)")
        Console.WriteLine("Press any key")

        Console.ReadKey()
        Console.WriteLine("")
        Console.WriteLine("")

        Try
            Dim amount = awContext.SaveChanges()
            Console.WriteLine("No conflict during saving. " _
                              & amount & " updates saved.")
        Catch ex As OptimisticConcurrencyException
            awContext.Refresh(RefreshMode.ClientWins, supplier)

            awContext.SaveChanges()
            Console.WriteLine("Conflict errors were caught. Changes were saved!")
        End Try

        'Save changes
        awContext.SaveChanges()

        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadKey()

    End Sub

End Module
