﻿Imports System.Data.Objects

Module SpecialQueryApproaches

    Sub Main()
        AvoidingAnonymousResultCollections()
        CompiledQueries()
    End Sub

    Sub AvoidingAnonymousResultCollections()

        Dim awContext As New AWEntities()

        'Queries all suppliers who have products 
        'whose names begins with "W" in the product details.
        Dim suppliers = From supplierItems In awContext.Vendor _
                        From supplierProductItem In supplierItems.ProductVendor _
                        From productItem In awContext.Product _
                            Where supplierItems.VendorID = supplierProductItem.VendorID _
                            Where productItem.ProductID = supplierProductItem.ProductID _
                            Where productItem.Name.StartsWith("W") _
                                Select supplierItems Distinct

        'Again, with this statement we can find out
        'which SQL text will be used.
        Console.WriteLine(CType(suppliers, ObjectQuery(Of Vendor)).ToTraceString)
        Console.WriteLine()

        'Use manual reloading for the output.
        For Each supplier In suppliers
            Console.WriteLine(supplier.AccountNumber & ": " & supplier.Name)
            Console.WriteLine(New String("="c, 70))

            If Not supplier.ProductVendor.IsLoaded Then
                supplier.ProductVendor.Load()
            End If

            'Deliver the products to each supplier
            For Each produkt In supplier.ProductVendor
                If Not produkt.ProductReference.IsLoaded Then
                    produkt.ProductReference.Load()
                    Console.Write("ID:" & produkt.ProductID & " ")
                    Console.WriteLine("Name:" & produkt.Product.Name)
                End If
            Next
            Console.WriteLine()
        Next

        Console.WriteLine()
        Console.WriteLine("Press key to exit.")
        Console.ReadKey()

    End Sub

    Sub CompiledQueries()

        Dim awContext As New AWEntities()

        Dim compiledSupplierQuery = CompiledQuery.Compile(Of AWEntities, _
                                                String, IQueryable(Of Vendor))( _
                        Function(context, FirstLetter) From supplier In context.Vendor _
                                Where supplier.Name.StartsWith(FirstLetter) _
                                Order By supplier.Name _
                                Select supplier)

        Dim queryFor_H = compiledSupplierQuery(awContext, "H")

        For Each supplier In queryFor_H
            Console.WriteLine(supplier.AccountNumber & ": " & supplier.Name)
            Console.WriteLine(New String("="c, 70))

            'Deliver the products to each supplier
            For Each product In supplier.ProductVendor
                Console.Write("ID:" & product.ProductID & " ")
                Console.WriteLine("Name:" & product.Product.Name)
            Next
            Console.WriteLine()
        Next

        Dim queryFor_W = compiledSupplierQuery(awContext, "W")

        For Each supplier In queryFor_W
            Console.WriteLine(supplier.AccountNumber & ": " & supplier.Name)
            Console.WriteLine(New String("="c, 70))

            'Deliver the products to each supplier
            For Each product In supplier.ProductVendor
                Console.Write("ID:" & product.ProductID & " ")
                Console.WriteLine("Name:" & product.Product.Name)
            Next
            Console.WriteLine()
        Next


        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadKey()

    End Sub

End Module