﻿Imports System.Threading
Imports System.Threading.Tasks

Module Module1

    Private myThreadCount As Integer
    Private myList As System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry(Of String))

    Sub Main()

        'CancelableVersion()
        'Return

        Dim Elements As New List(Of String) From {"Peter", "Ramona", "Lisa", "Sarika", "Jürgen",
                                          "Adriana", "Marcus", "Andreas", "Gabriele",
                                          "Silja", "Uta", "Oli", "Cate", "Arnold"}

        myList = New System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry(Of String))

        Dim sw = Stopwatch.StartNew
        Dim iterator As Integer
        Parallel.ForEach(Elements,
                         Sub(element As String)
                             iterator += 1
                             myList.Add(New ItemLogEntry(Of String)() With {.TickTime = sw.ElapsedMilliseconds,
                                                               .value = element,
                                                               .threadid = Thread.CurrentThread.ManagedThreadId})

                             'Pretend it takes a long time. The more
                             'elements the longer it takes.

                             Console.Write(".")
                             Thread.SpinWait(iterator * 2000000)
                         End Sub)

        Console.WriteLine()
        Console.WriteLine("Sorted by Thread-ID:")
        Dim sortedById = From item In myList
                     Order By item.ThreadId, item.TickTime

        sortedById.ToList.ForEach(Sub(item)
                                      Console.WriteLine(item.ToString)
                                  End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

        Console.WriteLine()
        Console.WriteLine("Sorted by Duration:")
        Dim sortedByTime = From item In myList
                     Order By item.TickTime

        sortedByTime.ToList.ForEach(Sub(item)
                                        Console.WriteLine(item.ToString)
                                    End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

    End Sub

    Public Structure ItemLogEntry(Of vType)
        Public TickTime As Long
        Public ThreadId As Long
        Public Value As vType

        Public Overrides Function ToString() As String
            Return String.Format("ThreadId {0:0000}: @{1:000000} ms - Value:{2}",
                                 ThreadId, TickTime, Value)
        End Function
    End Structure

    'IMPORTANT: In the book, the wrong listing has been printed!
    'This is the right version - sorry for that!
    Public Sub CancelableVersion()

        Dim Elements As New List(Of String) From {"Peter", "Ramona", "Lisa", "Sarika", "Jürgen",
                                  "Adriana", "Marcus", "Andreas", "Gabriele",
                                  "Silja", "Uta", "Oli", "Cate", "Arnold"}

        Dim flagWhichControlsThatAllValuesBelowTheCurrentGetProcessesed = False

        myList = New System.Collections.Concurrent.ConcurrentBag(Of ItemLogEntry(Of String))

        Dim sw = Stopwatch.StartNew
        Dim iterator As Integer
        Parallel.ForEach(Elements,
                         Sub(element As String, parLoopState As ParallelLoopState)
                             iterator += 1
                             myList.Add(New ItemLogEntry(Of String)() With {.TickTime = sw.ElapsedMilliseconds,
                                                               .value = element,
                                                               .threadid = Thread.CurrentThread.ManagedThreadId})

                             'Pretend it takes a long time. The more
                             'elements the longer it takes.
                             Console.Write(".")
                             Thread.SpinWait(iterator * 2000000)

                             'Exit if we found 'Jürgen'!
                             If element = "Jürgen" Then
                                 If flagWhichControlsThatAllValuesBelowTheCurrentGetProcessesed Then
                                     'Every parallel iteration which processes values "below" 'Jürgen' will 
                                     'continue until there are only elements left above which follow 'Jürgen' in the collection.
                                     parLoopState.Break()
                                 Else
                                     'Alternatively, with stop, all iterations will stop immediately.
                                     parLoopState.Stop()
                                 End If
                                 Console.Write("Exited for!")
                             End If
                         End Sub)

        Console.WriteLine()
        Console.WriteLine("Sorted by Thread-ID:")
        Dim sortedById = From item In myList
                     Order By item.ThreadId, item.TickTime

        sortedById.ToList.ForEach(Sub(item)
                                      Console.WriteLine(item.ToString)
                                  End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

        Console.WriteLine()
        Console.WriteLine("Sorted by Duration:")
        Dim sortedByTime = From item In myList
                     Order By item.TickTime

        sortedByTime.ToList.ForEach(Sub(item)
                                        Console.WriteLine(item.ToString)
                                    End Sub)
        Console.WriteLine("Return")
        Console.ReadLine()

    End Sub
End Module
