Imports System.Threading
Imports System.Threading.Tasks

Public Class frmMain

    Private Const LOOPCOUNT = 100

    'Member-Variable, so we can serially number the threads.
    'This is just for later examining purposes of the executing thread,
    'if it shows the results in the output window.
    Private myWorkerThreadNo As Integer = 1

    'This is the actual worker thread that takes over the
    'counting and value output
    Private Sub ComplexCalculation()

        Thread.CurrentThread.Name = "Worker-Thread: " + myWorkerThreadNo.ToString

        For c As Integer = 0 To LOOPCOUNT

            'Displays the value. TSWriteLine is a static procedure
            'that displays the window
            'if used for the first time.

            TextOutput.TSWriteLine(Thread.CurrentThread.Name + ": " + c.ToString)

            'The current thread is delayed by 100ms so the
            'execution doesn't happen too fast.
            Thread.Sleep(100)
        Next

    End Sub

    Private Sub btnStartThread_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                        Handles ThreadWithThreadButton.Click

        'This object encapsulates the actual thread
        Dim locThread As Thread

        'You need this object to determine the procedure
        'that runs the thread.
        Dim locThreadStart As ThreadStart

        'Procedure to run the thread
        locThreadStart = New ThreadStart(AddressOf ComplexCalculation)

        'ThreadStart object is passed to the thread
        locThread = New Thread(locThreadStart)

        'Create a thread that will end when the main application ends:
        locThread.IsBackground = True

        'Start thread
        locThread.Start()

        'Counter to distinguish the threads by their name property.
        myWorkerThreadNo += 1

    End Sub

    'Since Visual Basic 2010 the compiler saves a lot of typing
    'with multi-statement Lambdas:
    Private Sub ThreadStartShortVersion()

        Dim locThread As New Thread(
                                Sub()
                                    'Same loop as in the 'Complex calculation' method.
                                    For c As Integer = 0 To LOOPCOUNT
                                        TextOutput.TSWriteLine(Thread.CurrentThread.Name + ": " + c.ToString)
                                        Thread.Sleep(100)
                                    Next
                                End Sub) With
                            {.IsBackground = True,
                             .Name = "Worker Thread: " + myWorkerThreadNo.ToString}

        locThread.Start()

        myWorkerThreadNo += 1

    End Sub

    Private Sub btnBeenden_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnQuit.Click
        Me.Close()
    End Sub

    Private Sub ASyncInvokeButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ASyncInvokeButton.Click

        Dim aDelegate As action = AddressOf ComplexCalculation
        Dim asyncres As IAsyncResult = Nothing

        asyncres = aDelegate.BeginInvoke(
            Sub(value As Object)
                'This code is executed
                'after the task is completed.

                TextOutput.TSWriteLine("Worker Thread done!")

                'Important: Always end with EndInvoke!
                aDelegate.EndInvoke(asyncres)
                TextOutput.TSWriteLine("EndInvoke done!")
            End Sub, New Object)

        myWorkerThreadNo += 1
    End Sub

    Private Sub ThreadWithTasksButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThreadWithTasksButton.Click

        Dim t = Task.Factory.StartNew(AddressOf ComplexCalculation)

        'Alternatively:
        Dim t2 = New Task(Sub()
                              'Same loop as in the 'Complex calculation' method'.
                              For c As Integer = 0 To LOOPCOUNT
                                  TextOutput.TSWriteLine(Thread.CurrentThread.Name + ":: " +
                                  c.ToString)
                                  Thread.Sleep(100)
                              Next
                          End Sub)
        t2.Start()

        t.ContinueWith(
            Sub()
                TextOutput.TSWriteLine("Worker Thread completed!")
            End Sub)

    End Sub

    Private Sub ThreadFromThreadpoolDirectly_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThreadFromThreadpoolDirectly.Click

        ThreadPool.QueueUserWorkItem(Sub(value)
                                         ComplexCalculation()
                                     End Sub)
    End Sub
End Class