﻿Imports System.ComponentModel
Imports System.ComponentModel.Design

Public Class ThreadSafeTextWindowComponent
    Inherits Component

    Private myHostingForm As Form
    Private mySyncInvoker As ISynchronizeInvoke
    Private myOutputPane As TheTextForm
    Private myText As String = ""
    Private myThreadSyncer As New Object

    ''' <summary>
    ''' Internal use - don't use directly.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
     EditorBrowsable(EditorBrowsableState.Advanced),
     Browsable(False)>
    Public Overrides Property Site() As ISite
        <DebuggerHidden()>
        Get
            Return MyBase.Site
        End Get

        Set(ByVal value As ISite)
            MyBase.Site = value
            If value Is Nothing Then
                Return
            End If

            Dim host As IDesignerHost = TryCast(value.GetService(GetType(IDesignerHost)), IDesignerHost)
            If host IsNot Nothing Then
                Dim componentHost As IComponent = host.RootComponent
                If TypeOf componentHost Is Form Then
                    HostingForm = TryCast(componentHost, Form)
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' Sets or gets the Parent-Form, which is assigned to this component.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>NOTE: This property is automatically set, when you drag this component to the form, 
    ''' and you're not supposed to change it afterwards.</remarks>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Visible),
     EditorBrowsable(EditorBrowsableState.Advanced),
     Browsable(False)>
    Public Property HostingForm As Form
        Get
            Return myHostingForm
        End Get
        Set(ByVal value As Form)
            If value IsNot myHostingForm Then
                myHostingForm = value
                mySyncInvoker = DirectCast(myHostingForm, ISynchronizeInvoke)
            End If
        End Set
    End Property

    Friend ReadOnly Property OutputPane As TheTextForm
        Get
            SyncLock myThreadSyncer
                If myOutputPane Is Nothing Then
                    Dim outputPaneShower = Sub()
                                               myOutputPane = New TheTextForm
                                               AddHandler myOutputPane.FormClosing,
                                                   Sub(Sender As Object, e As FormClosingEventArgs)
                                                       myOutputPane = Nothing
                                                   End Sub
                                               myOutputPane.Show()
                                           End Sub

                    If mySyncInvoker IsNot Nothing Then
                        If mySyncInvoker.InvokeRequired Then
                            mySyncInvoker.Invoke(outputPaneShower, {})
                        Else
                            outputPaneShower()
                        End If
                    End If
                End If
                Return myOutputPane
            End SyncLock
        End Get
    End Property

    'Uses TSWrite; see there.
    Public Sub TSWriteLine(ByVal Message As String)
        Message += vbNewLine
        TSWrite(Message)
    End Sub

    'Print out without starting a new line.
    Public Sub TSWrite(ByVal Message As String)

        Dim outputDeL = Sub()
                            OutputPane.OutputTextBox.AppendText(Message)
                            OutputPane.OutputTextBox.SelectionStart = OutputPane.OutputTextBox.Text.Length - 1
                            OutputPane.OutputTextBox.ScrollToCaret()
                        End Sub

        'If the UI-Thread uses TSWrite we must not lock, because otherwise
        'Invoke can't be called inside the lock, since the UI thread would
        'be in charge to actually execute what's "inside" of Invoke,
        'If the UI-Thread would then again wait at this point...
        If Not Me.OutputPane.InvokeRequired Then
            outputDeL()
        Else
            Try
                Me.OutputPane.Invoke(outputDeL)
            Catch ex As Exception
                Debug.Print(ex.Message)
            End Try
        End If
    End Sub
End Class

Friend Class TheTextForm
    Inherits Form

    Private myOutputTextBox As TextBox

    Sub New()
        MyBase.New()
        myOutputTextBox = New TextBox
        With myOutputTextBox
            .Dock = DockStyle.Fill
            .ReadOnly = True
            .ScrollBars = ScrollBars.Vertical
            .Multiline = True
        End With
        Me.Controls.Add(myOutputTextBox)
    End Sub

    Friend ReadOnly Property OutputTextBox As TextBox
        Get
            Return myOutputTextBox
        End Get
    End Property
End Class
